!###############################################################################
!
! NAME
!   go_SystemMod  -  machine and/or compiler specific stuff
!
! DESCRIPTION
!   The module go_SystemMod provides some basic constants for the
!   current compiler. In addition, some interfaces are defined
!   to routines for system calls, setting of exit statuses etc, 
!   which are non-standard Fortran, but often provided by the
!   vendor of the compiler. 
!   Since both constants and system routines differ from compiler
!   to compiler, this GO module is available in a number of copies,
!   each valid for a single compiler. If for some compiler a 
!   certain constant or system routine could not be filled,
!   a dummy value is used or a warning is issued.
!
!   The following system routines are defined:
!
!    o call goSystem( command, status )
!        Perform a system command, return exit status.
!
!    o call goExit( status )
!        Stop execution, set the exit status.
!
!    o n = goArgC()
!        Count number of command line arguments.
!
!    o call goGetArg( nr, value )
!        Returns command line argument 'nr' in character string 'value'.
!
!###############################################################################
!
#define IF_ERROR_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!###############################################################################

module GO_System

  implicit none

  ! --- in/out ------------------------------
  
  private
  
  public   ::  goSystem
  public   ::  goExit
  public   ::  goArgC, goGetArg
  
  
  ! --- const ---------------------------------
  
  ! module name
  character(len=*), parameter  ::  mname = 'GO_System'
  

contains



  ! ############################################################################
  ! ###
  ! ### goSystem
  ! ###
  ! ############################################################################


  ! Execute a system command, return exit status.
  
  subroutine goSystem( command, status )
  
    use GO_Print, only : gol, goPr, goErr

#ifdef __INTEL_COMPILER
    use IFPort, only : System
    use IFPort, only : iErrNo, E2BIG, ENOENT, ENOEXEC, ENOMEM
#endif
  
    ! --- in/out -----------------------------------------------
    
    character(len=*), intent(in)       ::  command
    integer, intent(inout)             ::  status
    
    ! --- const ------------------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/goSystem'
    
    ! --- local --------------------------------------------------
    
#ifdef __INTEL_COMPILER
    integer(4)     ::  stat
    integer(4)     ::  errno
#endif

    ! --- begin --------------------------------------------------
    
    ! trap old errors
    if (status>0) return

#ifdef __INTEL_COMPILER

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! Intel Compiler
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    stat = System( command )
    
    ! trap errors in ifort system command
    if ( stat == -1 ) then
      write (gol,'("error in call to IFort Portability command `system`:")'); call goErr
      errno = iErrNo()
      select case ( errno )
        case ( E2BIG )
          write (gol,'("  ",a)') 'The argument list is too long.'; call goErr
        case ( ENOENT )
          write (gol,'("  ",a)') 'The command interpreter cannot be found.'; call goErr
        case ( ENOEXEC )
          write (gol,'("  ",a)') 'The command interpreter file has an invalid format and is not executable.'; call goErr
        case ( ENOMEM )
          write (gol,'("  ",a)') 'Not enough system resources are available to execute the command.'; call goErr
        case default
          write (gol,'("  unknown iErrNo ",i)') errno; call goErr
      end select
      write (gol,'("in ",a)') rname; call goErr; status=stat; return
    end if
    
    ! if the shell command exit status is 'n',
    ! then the number returned by 'system' is 256 * n
    status = stat / 256

#else

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! error ...
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    write (gol,'("subroutine not implemented for this compiler")'); call goErr
    write (gol,'("in ",a)') rname; call goErr; status=1; return

#endif
  
  end subroutine goSystem
  
  
  ! ############################################################################
  ! ###
  ! ### goExit
  ! ###
  ! ############################################################################


  ! Stop execution, set exit status.
  
  subroutine goExit( status )
  
    use GO_Print, only : gol, goPr, goErr

    ! --- in/out --------------------------------------------
    
    integer, intent(in)    ::  status
    
    ! --- const ------------------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/goExit'
    
    ! --- external -----------------------------------------
    
#ifdef __INTEL_COMPILER
    ! Intel (R) Fortran Language Reference
    !   9 Intrinsic Procedures
    !       Exit
    interface 
      subroutine Exit( status ) 
        integer(4), optional, intent(in) :: status 
      end subroutine 
    end interface
#endif
    
    ! --- begin --------------------------------------------
    
#ifdef __INTEL_COMPILER

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! Intel Compiler
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    call Exit( int(status,kind=4) )
    
#else

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! error ...
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    write (gol,'("subroutine not implemented for this compiler")'); call goErr
    write (gol,'("in ",a)') rname; call goErr; 
    
    ! this is an emergency, so for one time, the Fortran stop is allowed ...
    stop 'Fortran STOP in GO_System/goExit'

#endif
  
  end subroutine goExit


  ! ############################################################################
  ! ###
  ! ### goArgC
  ! ###
  ! ############################################################################


  ! Return number of command line arguments
  
  integer function goArgC()
  
    use GO_Print, only : gol, goPr, goErr

    ! --- const ------------------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/goArgC'
    
    ! --- external ----------------------------------------------

#ifdef __INTEL_COMPILER
    ! Intel (R) Fortran Language Reference
    !   9 Intrinsic Procedures
    !       iArgC
    interface 
      integer(4) function iArgC() 
      end function iArgC
    end interface
#endif
    
    ! --- begin -------------------------------------------------
    
#ifdef __INTEL_COMPILER

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! Intel Compiler
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    goArgC = iArgC()
    
#else

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! error ...
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    write (gol,'("subroutine not implemented for this compiler")'); call goErr
    write (gol,'("in ",a)') rname; call goErr
    
    ! always assign something ...
    goArgC = -1

#endif
  
  end function goArgC
  
  
  ! ############################################################################
  ! ###
  ! ### goGetArg
  ! ###
  ! ############################################################################


  ! Return a command line argument
  
  subroutine goGetArg( pos, value, status )
  
    use GO_Print, only : gol, goPr, goErr

    ! --- in/out --------------------------------------------------
    
    integer, intent(in)             ::  pos
    character(len=*), intent(out)   ::  value
    integer, intent(inout)          ::  status
    
    ! --- const ------------------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/goGetArg'
    
    ! --- external ------------------------------------------------
    
    ! trap old errors
    if (status>0) return

#ifdef __INTEL_COMPILER
    ! Intel (R) Fortran Language Reference
    !   9 Intrinsic Procedures
    !       GetArg
    !
    !  interface GetArg 
    !    subroutine GetArg( n, buffer, status ) 
    !      integer, intent(in)                :: n       ! kind is 2 or 4
    !      character(len=*), intent(out)      :: buffer 
    !      integer, intent(out), optional     :: status  ! kind as n
    !    end subroutine 
    !  end interface 
    !
    ! NOTE: for some reason, uncommenting the interface
    !   lets the routine fail ...
    !
#endif
      
    ! --- begin -----------------------------------------------------

    ! trap old errors
    if (status>0) return

#ifdef __INTEL_COMPILER

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! Intel Compiler
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    ! call system routine:    
    call GetArg( pos, value, status )

    ! Number of characters set in buffer is returned in stat:
    if ( status <= 0 ) then
      status = 1
    else
      status = 0
    end if
    
#else

    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ! error ...
    ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    write (gol,'("subroutine not implemented for this compiler")'); call goErr
    write (gol,'("in ",a)') rname; call goErr; return

    ! always assign something ...
    value = '?'
    status = 1

#endif
  
  end subroutine goGetArg


end module GO_System


