!
! Parallelisation stuff.
! Dummy variables and routines for single processor case.
!
! npes                 |     1    |             3
! ---------------------+----------+-----------------------------------------
! myid                 |     0    |       0            1            2
!                      |          |
! root                 |     0    |       0            0            0
! root_k               |     0    |       0            0            0
! root_t               |     0    |       0            0            0
!                      |          |
! lm                   |    25    |      25           25           25
! lmloc                |    25    |       8            8            9
! lmar(0:npes-1)       |  (/25/)  |   (/8,8,9/)    (/8,8,9/)    (/8,8,9/)
! offsetl              |     0    |       0            8           16
!                      |          |
! ntrace               |    42    |      42           42           42
!                      |          |
! ntracet              |    26    |      26           26           26
! ntracetloc           |    26    |       8            9            9
! ntracet_ar(0:npes-1) |  (/26/)  |   (/8,9,9/)    (/8,9,9/)    (/8,9,9/)
!                      |          |
! tracer_active(1:26)  |    T  1  |     T  1          F           F   
! tracer_loc(1:26)     |    :  :  |     :             :           :
!                      |    T  8  |     T  8          F           F 
!                      |    T  9  |     F             T  1        F 
!                      |    :  :  |     :             :  :        :
!                      |    T 17  |     F             T  9        F
!                      |    T 18  |     F             F           T  1
!                      |    :  :  |     :             :           :  :
!                      |    T 26  |     F             F           T  9
!                      |          |
!
!
!  Error handling in MPI programs
!  -----------------------------------
!
!   Change the error handler used by a communicator:
!
!      subroutine MPI_ErrHandler_Set( comm, ErrHandler, iError )
!        integer, intent(in)    ::  comm
!        integer, intent(in)    ::  ErrHandler
!        integer, intent(in)    ::  iError
!      end subroutine MPI_ErrHandler_Set
!
!  Standard error handlers :
!      MPI_ERRORS_ARE_FATAL       ! default
!      MPI_ERRORS_RETURN 
!
!



!
!

!### macro's #####################################################

#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if

!#################################################################
  
  
module ParTools

  use GO        , only : gol, goErr
#ifdef MPI
  use mpi_const , only : npes, myid, maxtrac_per_pe
  use mpi_const , only : root, root_k, root_t
  use mpi_const , only : tracer_active, tracer_loc, ntracetloc, ntracet_ar, proc_tracer
  use mpi_const , only : lmloc, lmar
  use mpi_const , only : which_par, previous_par
  use mpi_const , only : allocate_mass
  use mpi_const , only : MPI_COMM_WORLD, MPI_INFO_NULL
  use mpi_const , only : nmembers, nmembers_per_pe, nmembersloc
#else
  use dims      , only : nregions
#endif 
  use chem_param, only : ntracet
  
  implicit none

  ! --- in/out -----------------------------------
  
  private
  
  public  ::  npes, myid,maxtrac_per_pe, proc_tracer
  public  ::  nmembers, nmembers_per_pe, nmembersloc
  public  ::  procname
  public  ::  root, root_k, root_t
  public  ::  lmloc, lmar, offsetl
  public  ::  ntracetloc, ntracet_ar
  public  ::  tracer_loc, tracer_active, tracer_id
  public  ::  which_par, previous_par
  public  ::  allocate_mass
  
  public  ::  MPI_COMM_WORLD, MPI_INFO_NULL
  public  ::  PAR_OPER_SUM

  public  ::  Par_Init, Par_Done
  public  ::  Par_Barrier
  public  ::  Par_StopMPI
  public  ::  Par_Broadcast
  public  ::  Par_Reduce, Par_AllReduce
  public  ::  Par_Gather_Tracer_t
  public  ::  Par_Gather_Tracer_k
  public  ::  Par_Scatter_After_Read_t
  public  ::  Par_Scatter_Over_Levels
  
  ! --- const --------------------------------------

  character(len=*), parameter  ::  mname = 'ParTools'

  ! operations 
  integer, parameter        ::  PAR_OPER_SUM = 100

  ! --- var --------------------------------------

#ifdef MPI
#else

  ! dummy constants:
  integer, parameter  ::  MPI_INFO_NULL = 0

  ! dummy communicator:
  integer, parameter  ::  MPI_COMM_WORLD = 0

  ! same as used from mpi const:
  integer,parameter ::  npes=1
  !integer,parameter ::  maxtrac_per_pe=ceiling(1.0*ntracet/npes)  ! number of tracers per pe maximum
  integer,parameter ::  maxtrac_per_pe=ntracet
  integer           ::  myid     ! PE number in mpi_comm_world (always one if not MPI)
  integer           ::  root     ! myid of root in mpi_comm_world
  integer           ::  root_k   ! myid of root in com_lev
  integer           ::  root_t   ! myid root in com_trac

  ! nr of levels at this PE
  integer               ::  lmloc

  ! number of levels actually assigned to each PE
  integer, allocatable  ::  lmar(:)
  
  ! nr of tracers and transported tracers at this PE
  integer               ::  ntracetloc    

  ! nr of transported tracers to each PE
  integer, allocatable  ::  ntracet_ar(:)

  ! tracer_active  : determines whether tracer is active on processer 
  logical           ::  tracer_active(ntracet)

  ! tracer_loc  : determines location in the local array 
  integer           ::  tracer_loc(ntracet)

  ! proc_tracer  : determines which PE handles each tracer
  integer,dimension(ntracet)    ::  proc_tracer

  ! parrallel regime : either 'levels' or 'tracer'
  character(len=6)        ::  which_par  
  character(len=6)        ::  previous_par(nregions)

  ! switch to allocate and deallocate mass after each swap
  logical, parameter      ::  allocate_mass = .false.

  ! EnKF variables
  integer                       :: nmembers         ! number of members total
  integer                       :: nmembers_per_pe  ! max number of members per pe
  integer                       :: nmembersloc      ! local number of members for each PE

#endif

  ! level offset: lglob = offsetl+lloc
  integer           ::  offsetl

  ! character keys for each processor
  character(len=6)  ::  procname
  
  ! processor id's for each tracer:
  integer           ::  tracer_id(ntracet)

  
  ! --- interfaces -----------------------------------
  
  interface Par_Broadcast
    module procedure Par_Broadcast_r2
    module procedure Par_Broadcast_r3
  end interface

  interface Par_Reduce
    module procedure Par_Reduce_r2
  end interface

  interface Par_Gather_Tracer_k
    module procedure Par_Gather_Tracer_k_3d
    module procedure Par_Gather_Tracer_k_4d
  end interface

  interface Par_Scatter_Over_Levels
    module procedure Par_Scatter_Over_Levels__r3
  end interface


contains


  ! ===================================================
  
  !
  ! Inititialisation.
  ! Copied from main program.
  !
  
  subroutine Par_Init( status )
  
    use Dims, only : lm
#ifdef MPI
    use mpi_comm, only : startmpi
#endif

    ! --- in/out ---------------------------------
    
    integer, intent(out)          ::  status
    
    ! --- local ----------------------------------
    
    integer        ::  id
    integer        ::  n
    
    ! --- begin ----------------------------------
    
#ifdef MPI

    call startmpi

#else

    ! single processor
    !npes = 1

    ! dummy id for root processor
    root = 0

    ! only one processor, so this is always root ...
    myid = root
    root_k = root
    root_t = root 

    ! *** levels

    ! allocate arrays
    allocate( lmar(0:npes-1) )

    ! nr of levels at this PE
    ! (take levels from region 1)
    lmloc = lm(1)

    ! number of levels actually assigned to each PE
    ! (take levels from region 1)
    lmar(0) = lm(1)
  
    ! *** tracers

    ! allocate arrays
    allocate( ntracet_ar(0:npes-1) )

    ! tracer_active  : determines whether tracer is active on processer 
    tracer_active = .true.
 
    ! number of transported tracers
    ntracetloc = ntracet
 
    ! tracer location
    tracer_loc = -9
    do n = 1, ntracet
      tracer_loc(n) = n
    end do
    
    ! single processor is 'parallel' over tracers:
    which_par = 'tracer'
    previous_par = 'tracer'

    ! processor and tracer
    proc_tracer(:) = 0
    
    ! ***

#endif

    ! level offset: lglob = offsetl+lloc
    offsetl = 0
    if ( myid > 0 ) offsetl = sum(lmar(0:myid-1))
    
    ! fill processor id for each tracer:
    tracer_id = -1
    do n = 1, ntracet
      if ( n <= ntracet_ar(0) ) then
        tracer_id(n) = 0
      else
        do id = 1, npes-1
          if ( (n > sum(ntracet_ar(0:id-1))) .and. &
               (n <= sum(ntracet_ar(0:id))) ) tracer_id(n) = id
        end do
      end if
    end do

    ! set processor names: pe0000, pe0001, ...
    write (procname,'("pe",i4.4)') myid

    ! ok
    status = 0

  end subroutine Par_Init
  
  
  ! ***
  
  
  subroutine Par_Done( status )
  
#ifdef MPI
    use mpi_comm, only : stopmpi
#endif
    
    ! --- in/out ------------------------------
    
    integer, intent(out)          ::  status
    
    ! --- begin ------------------------------

#ifdef MPI
    
    !CMK ???? call stopmpi
    
#else
  
    ! deallocate arrays
    deallocate( lmar       )
    deallocate( ntracet_ar )

#endif
    
    ! ok
    status = 0

  end subroutine Par_Done
  

  ! ***
  
  
  subroutine Par_Barrier

#ifdef MPI
    use mpi_comm, only : barrier
#endif

    ! --- begin ------------------------------------
      
#ifdef MPI   
    call barrier
#endif

  end subroutine Par_Barrier
    


  ! ***
  
  
  subroutine Par_StopMPI

#ifdef MPI
    use mpi_comm, only : stopmpi
#endif

    ! --- begin ------------------------------------
      
#ifdef MPI   

    call stopmpi

#endif

  end subroutine Par_StopMPI


  ! **************************************************************
  

  subroutine Par_Broadcast_r2( x, id, status )
  
#ifdef MPI
    use mpi_const, only : MPI_REAL8, MPI_COMM_WORLD
#endif

    ! --- in/out -------------------------------------
    
    real(8), intent(inout)   ::  x(:,:)
    integer, intent(in)      ::  id
    integer, intent(out)     ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Broadcast_r2'
    
    ! --- begin ------------------------------------

    if ( npes > 1 ) then
#ifdef MPI
      call mpi_bcast( x, size(x), MPI_REAL8, id, MPI_COMM_WORLD, status )
      if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
#else
      write (*,'("ERROR - please implement for non-mpi parallel library.")')
      write (*,'("ERROR in ",a)') rname; status=1; return
#endif
    end if
    
    ! ok
    status = 0
    
  end subroutine Par_Broadcast_r2


  ! ***
  
  
  subroutine Par_Broadcast_r3( x, id, status )
  
#ifdef MPI
    use mpi_const, only : MPI_REAL8, MPI_COMM_WORLD
#endif

    ! --- in/out -------------------------------------
    
    real(8), intent(inout)   ::  x(:,:,:)
    integer, intent(in)      ::  id
    integer, intent(out)     ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Broadcast_r3'
    
    ! --- begin ------------------------------------

    if ( npes > 1 ) then
#ifdef MPI
      call mpi_bcast( x, size(x), MPI_REAL8, id, MPI_COMM_WORLD, status )
      if (status/=0) then; write (*,'("ERROR in ",a)') rname; status=1; return; end if
#else
      write (*,'("ERROR - please implement for non-mpi parallel library.")')
      write (*,'("ERROR in ",a)') rname; status=1; return
#endif
    end if
    
    ! ok
    status = 0
    
  end subroutine Par_Broadcast_r3


  ! *********************************************************
  
  
  subroutine Par_AllReduce( x_l, x, command, status )
  
#ifdef MPI
    use mpi_const, only : MPI_MAX
    use mpi_const, only : MPI_REAL8, MPI_COMM_WORLD
#endif

    ! --- in/out -------------------------------------
    
    real(8), intent(in)               ::  x_l
    real(8), intent(out)              ::  x
    character(len=*), intent(in)      ::  command
    integer, intent(out)              ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_AllReduce'
    
    ! --- local -----------------------------------
    
    integer           ::  icommand
    
    ! --- begin ------------------------------------

    if ( npes > 1 ) then

#ifdef MPI
      
      ! determine correct mpi constant
      select case ( command )
        case ( 'max', 'MAX' )
          icommand = MPI_MAX
        case default
          write (*,'("ERROR - unsupported command for mpi : ",a)') command
          write (*,'("ERROR in ",a)') rname; status=1; return
      end select

      ! reduce local values to global value:
      call MPI_AllReduce( x_l, x, 1, MPI_REAL8, &
                          icommand, MPI_COMM_WORLD, status ) 
      if (status/=0) then
        write (*,'("ERROR - error from MPI_AllReduce : ",i6)') status
        write (*,'("ERROR in ",a)') rname; status=1; return
      end if

#else

      write (*,'("ERROR - please implement for non-mpi parallel library.")')
      write (*,'("ERROR in ",a)') rname; status=1; return

#endif

    else
    
      ! only one processor; just copy:
      x = x_l

    end if
    
    ! ok
    status = 0
    
  end subroutine Par_AllReduce


  ! *********************************************************
  
  
  !
  ! gather an array  
  !    tracer(1-ih,im+ih,1-jh,jm+jh,1-lh,lm+lh,ntracer) 
  ! on root_t
  ! from local arrays  
  !    tracerloc(1-ih,im+ih,1-jh,jm+jh,1-lh:lm+lh,ntracerloc) 
  ! on all PE's where  
  ! the array is distributed over the n-index
  !

  subroutine Par_Gather_Tracer_t( tracer, im,jm,lm, ih,jh,lh, ntracer, tracerloc, broadcast )

#ifdef MPI   
    use mpi_comm, only : gather_tracer_t
#endif
    
    ! --- in/out ----------------------------------
    
    real, intent(inout)    ::  tracer(:,:,:,:)  
    integer, intent(in)    ::  im,jm,lm
    integer, intent(in)    ::  ih,jh,lh
    integer, intent(in)    ::  ntracer
    real, intent(in)       ::  tracerloc(:,:,:,:)
    logical, intent(in)    ::  broadcast

    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/ Par_Gather_Tracer_t'
    
    ! --- begin -----------------------------------

#ifdef MPI   
    call gather_tracer_t( tracer, im,jm,lm, ih,jh,lh, ntracer, tracerloc, broadcast )
#else
    tracer = tracerloc
#endif

  end subroutine Par_Gather_Tracer_t


  ! *********************************************************
  
  
  subroutine Par_Gather_Tracer_k_3d( tracer, im,jm,lm, ih,jh,lh, &
                                       tracerloc, broadcast, status )

#ifdef MPI
    use mpi_comm, only : Gather_Tracer_k_3d
#endif

    ! --- in/out -----------------------------
    
    real, intent(inout)       ::  tracer(:,:,:)
    integer, intent(in)       ::  im,jm,lm
    integer, intent(in)       ::  ih,jh,lh
    real, intent(in)          ::  tracerloc(:,:,:)
    logical, intent(in)       ::  broadcast
    integer, intent(out)      ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Gather_Tracer_k_3d'
    
    ! --- begin --------------------------------
    
    if ( npes == 1 ) then
    
      ! just copy ...
      tracer = tracerloc
      
    else
    
#ifdef MPI

    call Gather_Tracer_k_3d( tracer, im,jm,lm, ih,jh,lh, tracerloc, broadcast )

#else

      write (gol,'("ERROR - please implement for non-mpi parallel library.")'); call goErr
      write (gol,'("ERROR in ",a)') rname; call goErr; status=1; return

#endif

    end if

    ! ok
    status = 0

  end subroutine Par_Gather_Tracer_k_3d


  ! ***
  

  subroutine Par_Gather_Tracer_k_4d( tracer, im,jm,lm, ih,jh,lh, ntracer, &
                                        tracerloc, broadcast, status )

#ifdef MPI
    use mpi_comm, only : Gather_Tracer_k
#endif

    ! --- in/out -----------------------------
    
    real, intent(inout)       ::  tracer(:,:,:,:)
    integer, intent(in)       ::  im,jm,lm
    integer, intent(in)       ::  ih,jh,lh
    integer, intent(in)       ::  ntracer
    real, intent(in)          ::  tracerloc(:,:,:,:)
    logical, intent(in)       ::  broadcast
    integer, intent(out)      ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Gather_Tracer_k_4d'
    
    ! --- begin --------------------------------
    
    if ( npes == 1 ) then
    
      ! just copy ...
      tracer = tracerloc
      
    else
    
#ifdef MPI

      call Gather_Tracer_k( tracer, im,jm,lm, ih,jh,lh, ntracer, tracerloc, broadcast )

#else

      write (gol,'("ERROR - please implement for non-mpi parallel library.")'); call goErr
      write (gol,'("ERROR in ",a)') rname; call goErr; status=1; return

#endif

    end if

    ! ok
    status = 0

  end subroutine Par_Gather_Tracer_k_4d


  ! *********************************************************
  

  !
  ! scatter an array
  !    tracer    (1-ih,im+ih,1-jh,jm+jh,1-lh,lm+lh,ntracer)
  ! on root (pe=0)
  ! over local arrays
  !    tracerloc(1-ih,im+ih,1-jh,jm+jh,1,  ,lmloc,ntracer) 
  ! on all PE's so that 
  ! the array is distributed over the l-index
  !

  subroutine Par_Scatter_After_Read_t( tracer_read, im,jm,lm, ih,jh,lh, tracerloc, send_id )

#ifdef MPI   
    use mpi_comm, only : Scatter_After_Read_t
#endif
    
    ! --- in/out -----------------------------------

    real, intent(in)            ::  tracer_read(:,:,:,:)
    integer,intent(in)          ::  im,jm,lm
    integer,intent(in)          ::  ih,jh,lh
    real, intent(out)           ::  tracerloc (:,:,:,:)  
    integer,intent(in)          ::  send_id 
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Scatter_After_Read_t'
    
    ! --- begin ---------------------------------
    
    if ( npes == 1 ) then
    
      ! just copy ...
      tracerloc = tracer_read
      
    else
    
#ifdef MPI

      call scatter_after_read_t( tracer_read, im,jm,lm, ih,jh,lh, tracerloc, send_id )

#else

      write (gol,'("ERROR - please implement for non-mpi parallel library.")'); call goErr
      write (gol,'("ERROR in ",a)') rname; call goErr; stop

#endif

    end if

  end subroutine Par_Scatter_After_Read_t


  ! ***
  
  
  !
  ! Distribute array 'x_send(:,:,lm)' on pe 'id_send'
  ! over arrays 'x_recv(:,:,lmloc)' on all pe's .
  !
  
  subroutine Par_Scatter_Over_Levels__r3( x_send, id_send, x_recv, status )
  
#ifdef MPI
    use mpi_const, only : MPI_REAL8
#endif

    ! --- in/out -------------------------------------
    
    real(8), intent(in)      ::  x_send(:,:,:)
    real(8), intent(out)     ::  x_recv(:,:,:)
    integer, intent(in)      ::  id_send
    integer, intent(out)     ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Scatter_Over_Levels__r3'

    integer, parameter  ::  ndim = 3
    
    ! --- local ------------------------------------

    integer           ::  sendcounts(0:npes-1)
    integer           ::  displ(0:npes-1)
    integer           ::  recvcount
    
    integer           ::  n
    
    ! --- begin ------------------------------------

    if ( npes == 1 ) then
    
      x_recv = x_send
      
    else

#ifdef MPI

      sendcounts = size(x_send,1) * size(x_send,2) * lmar

      displ(0) = 0
      do n = 1, npes-1
        displ(n) = sum(sendcounts(0:n-1))
      end do
      
      recvcount  = size(x_recv,1) * size(x_recv,2) * lmloc

      call MPI_SCATTERV( x_send, sendcounts, displ, MPI_REAL8, & 
                         x_recv, recvcount        , MPI_REAL8, &
                         id_send, MPI_COMM_WORLD, status )
      IF_NOTOK_RETURN(status=1)

#else

      write (*,'("ERROR - please implement for non-mpi parallel library.")')
      write (*,'("ERROR in ",a)') rname; status=1; return

#endif

    end if
    
    ! ok
    status = 0
    
  end subroutine Par_Scatter_Over_Levels__r3



  ! =========================================================================
  
    
  
  subroutine Par_Reduce_r2( send, recv, oper, id_recv, status )

#ifdef MPI  
    use mpi_const, only : MPI_SUCCESS, MPI_SUM
    use mpi_const, only : my_real
#endif
  
    ! --- in/out -----------------------------
    
    real, intent(in)          ::  send(:,:)
    real, intent(out)         ::  recv(:,:)
    integer, intent(in)       ::  oper
    integer, intent(in)       ::  id_recv
    integer, intent(out)      ::  status
    
    ! --- const -----------------------------
    
    character(len=*), parameter  ::  rname = mname//'/Par_Reduce_r2'
    
    ! --- local -----------------------------
    
    integer         ::  mpi_op
    integer         ::  slen
    
    ! --- begin ------------------------------
    
    ! parallel ?
    if ( npes == 1 ) then
    
      ! just copy
      recv = send
      
    else
    
#ifdef MPI

      ! select mpi reduce operation:
      select case ( oper )
        case ( PAR_OPER_SUM ) ; mpi_op = MPI_SUM
        case default
          write (gol,'("unsupported oper : ",i6)') oper; call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
      end select

      ! reduce to processor id_send
      call MPI_Reduce( send, recv, size(send), MY_REAL, &
                          mpi_op, id_recv, MPI_COMM_WORLD, status )
      if (status/=MPI_SUCCESS) then
        call MPI_Error_String( status, gol, slen ); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
      end if

#else

      write (gol,'("please implement for non mpi library")'); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
        
#endif

    end if  ! parallel ?

    ! ok
    status = 0
    
  end subroutine Par_Reduce_r2
  


end module ParTools
